#include "chart.h"
#include <QtWidgets/QGesture>
#include <QtWidgets/QGraphicsScene>
#include <QtWidgets/QGraphicsView>
#include <QLineSeries>
#include <iostream>

Chart::Chart(QGraphicsItem *parent, Qt::WindowFlags wFlags)
    : QChart(QChart::ChartTypeCartesian, parent, wFlags)
{
    // Seems that QGraphicsView (QChartView) does not grab gestures.
    // They can only be grabbed here in the QGraphicsWidget (QChart).
    grabGesture(Qt::PinchGesture);
    grabGesture(Qt::PanGesture);
}

Chart::~Chart() {

}

bool Chart::sceneEvent(QEvent *event) {
    if (event->type() == QEvent::Gesture)
        return gestureEvent(static_cast<QGestureEvent *>(event));
    return QChart::event(event);
}

bool Chart::gestureEvent(QGestureEvent *event) {
    if (QGesture *gesture = event->gesture(Qt::PanGesture)) {
        QPanGesture *pan = static_cast<QPanGesture *>(gesture);
        QChart::scroll(-(pan->delta().x()), pan->delta().y());
    }
    if (QGesture *gesture = event->gesture(Qt::PinchGesture)) {
        QPinchGesture *pinch = static_cast<QPinchGesture *>(gesture);
        if (pinch->changeFlags() & QPinchGesture::ScaleFactorChanged)
            QChart::zoom(pinch->scaleFactor());

    }
    return true;
}

void Chart::XRangeChanged(qreal min, qreal max) {
    std::cerr << "XRangeChanged" << min <<  "\t"<< max << std::endl;
    if (series().at(0)->type() == QtCharts::QAbstractSeries::SeriesTypeLine) {
        QLineSeries *s = static_cast<QLineSeries *>(series().at(0));
        std::cerr << s->at(0).x() << "\t"<< s->at(s->count()-1).x() << std::endl;
    }
}
